/***********************************************************************************************************************
file:                   Display.c
description:            High level functions to work with graphic displays
author:                 VDLab
***********************************************************************************************************************/
#include "Display.h"
unsigned char FRED;
unsigned char FGREEN;
unsigned char FBLUE;

unsigned char BRED;
unsigned char BGREEN;
unsigned char BBLUE;
/*----------------------------------------------------------------------------------------------------------------------
function:               SetColor
description:            set current foreground and background colors
input:                  Forecol - current foreground color
                        BackCol - current background color
output:                 none
----------------------------------------------------------------------------------------------------------------------*/
void SetColor(unsigned int ForeCol, unsigned int BackCol)
{
FRED = ((ForeCol>>8)&0xF8);
FGREEN = ((ForeCol>>3)&0xFC);
FBLUE = ((ForeCol<<3)&0xF8);

BRED = ((BackCol>>8)&0xF8);
BGREEN = ((BackCol>>3)&0xFC);
BBLUE = ((BackCol<<3)&0xF8);
}
/*----------------------------------------------------------------------------------------------------------------------
function:               Line
description:            draw line by Bresenham algorithm 
input:                  Xstart,Ystart - line start coordinates
                        Xend,Yend - line end coordinates
output:                 none
----------------------------------------------------------------------------------------------------------------------*/
void Line(int Xstart, int Ystart, int Xend, int Yend)
{
unsigned int nTmp;
unsigned int nAlt=0;
int x,y;                                    // where is the current pixel.
int dx;   	                                // dx is the delta for x
int dy;   	                                // dy is the delta for y
int StepVal=0;                              // variable for figuring out when to increment the other axis.
if (Xstart>Xend && Ystart>Yend)
    {
	nTmp=Xend;
	Xend=Xstart;
	Xstart=nTmp;
	nTmp=Yend;
	Yend=Ystart;
	Ystart=nTmp;
	
	dx=Xend-Xstart;                         // dx is the delta for x
	dy=Yend-Ystart;                         // dy is the delta for y
    }
else
    {
	dx=Xend-Xstart;                         // dx is the delta for x
	dy=Yend-Ystart;                         // dy is the delta for y
	if (dy<0)
        {	
		dy=-dy;
		nTmp=Yend;
		Yend=Ystart;
		Ystart=nTmp;			
		nAlt=1;
        }
	else
		if (dx<0)
			{	
			dx=-dx;
			nTmp=Xend;
			Xend=Xstart;
			Xstart=nTmp;
			nAlt=1;
			}
	}
	if (nAlt)
	    {
        if(dx>=dy)                          // The slope is less than 45 degres
            {
			y=Yend;
			for(x=Xstart; x<=Xend; x++)
                {
				PutPixel(x, y);
				StepVal+=dy;
				if(StepVal>=dx)             // Increment y if enough x steps have been taken.
                    {
					y--;
					StepVal-=dx;            // Reset StepVal, but not to 0.  This gives even slopes.
                    }
                }
            }
		else                                // The slope is greater than 45 degrees, just like above, but with y instead of x.
            {
			x=Xend;
			for(y=Ystart; y<=Yend; y++)
                {
				PutPixel(x, y);
				StepVal+=dx;
				if(StepVal>=dy)
                    {
					x--;
					StepVal-=dy;
                    }
                }
            }
		return;
        }
	if(dx>=dy)                              // The slope is less than 45 degres
        {
		y=Ystart;
		for(x=Xstart; x<=Xend; x++)
            {
			PutPixel(x, y); 
			StepVal+=dy;
			if(StepVal>=dx)                 // Increment y if enough x steps have been taken.
                {
				y++;
				StepVal-=dx;                // Reset StepVal, but not to 0.  This gives even slopes.
                }
            }
        }
	else                                    // The slope is greater than 45 degrees, just like above, but with y instead of x.
        {
		x=Xstart;
		for(y=Ystart; y<=Yend; y++)
            {
			PutPixel(x, y);
			StepVal+=dx;
			if(StepVal>=dy)
                {
				x++;
				StepVal-=dy;
                }
            }
        }
	return;
}
/*----------------------------------------------------------------------------------------------------------------------
function:               Circle
description:            draw circle by Bresenham algorithm 
input:                  x,y - center coordinates
                        r - radius of circle
output:                 none
----------------------------------------------------------------------------------------------------------------------*/
void Circle(int x, int y, int r)
{
int x1,y1,yk = 0;
int sigma,delta,f;

x1 = 0;
y1 = r;
delta = 2*(1-r);
do
    {
    PutPixel(x+x1,y+y1);
    PutPixel(x-x1,y+y1);
    PutPixel(x+x1,y-y1);
    PutPixel(x-x1,y-y1);
    f = 0;
    if (y1 < yk) break;
    if (delta < 0)
        {
        sigma = 2*(delta+y1)-1;
        if (sigma <= 0)
            {
            x1++;
            delta += 2*x1+1;
            f = 1;
            }
        }
    else
        if (delta > 0)
            {
            sigma = 2*(delta-x1)-1;
            if (sigma > 0)
                {
                y1--;
                delta += 1-2*y1;
                f = 1;
                }
            } 
        if (!f)
            {
            x1++;
            y1--;
            delta += 2*(x1-y1-1);
            }
        }
    while(1);
}
/*----------------------------------------------------------------------------------------------------------------------
function:               Grid
description:            draw scope like grid
input:                  type - if =1 - draws center of screen lines 
output:                 none
----------------------------------------------------------------------------------------------------------------------*/
void Grid(char type)
{
unsigned int x, y;

if (type)
    {
    Line(0,120,319,120);
    Line(160,1,160,238);
    }
for(y=0;y<240;y+=20)
	{
	for(x=0;x<320;x+=20)
		{
		PutPixel(x,y);
		}
	}
Line(0,0,319,0);
Line(0,1,0,239);
Line(1,239,319,239);
Line(319,1,319,238);
}
/*----------------------------------------------------------------------------------------------------------------------
function:               PutStr
description:            put string to display
input:                  x,y - position of text start
                        *str - pointer to string to output
output:                 none
----------------------------------------------------------------------------------------------------------------------*/
void PutStr(unsigned int x, unsigned int y, char *str)
{
	char cmprr;
	
while ((cmprr = *str++))
    {
	PutChar(x,y,cmprr);
	x+=8;
	if (x>311) return; 
	}
}
/*----------------------------------------------------------------------------------------------------------------------
function:               Rectangle
description:            draw rectangle
input:                  x,y - upper left corner coordinates
                        width - width of rectangle
                        height -height of rectangle
                        Filled - if =1 - rectangle is filled
output:                 none
----------------------------------------------------------------------------------------------------------------------*/
void Rectangle(unsigned int X, unsigned int Y, unsigned int Width, unsigned int Height,unsigned char Filled)
{
	unsigned long tmp;
	
SetWindow(X,Y,X+(Width-1),Y+(Height-1));
if (Filled)
	{
	tmp = Height*Width;
	pinDC = 0;
	PMDIN1 = 0x2C;
	pinDC = 1;
	do  {
    	PMDIN1 = FRED;
    	PMDIN1 = FGREEN;
    	PMDIN1 = FBLUE;
    	}while(tmp--);
	}
else
	{
	Line(X,Y,X+(Width-1),Y);
	Line(X,Y+1,X,Y+(Height-1));
	Line(X+1,Y+(Height-1),X+(Width-1),Y+(Height-1));
	Line(X+(Width-1),Y+1,X+(Width-1),Y+(Height-1));
	}
}
/*----------------------------------------------------------------------------------------------------------------------
function:               Window
description:            draw windows95 like window 
input:                  x,y - upper left corner coordinates
                        width - width of window
                        height -height of window
                        header - pointer to window header string 
output:                 none
----------------------------------------------------------------------------------------------------------------------*/
void Window(unsigned int X, unsigned int Y, unsigned int Width, unsigned int Height, char *header)
{
SetColor(0xC618,0);
Line(X,Y,X+(Width-1),Y);
Line(X,Y+1,X,Y+(Height-1));
SetColor(0x630C,0);
Line(X+1,Y+(Height-1),X+(Width-1),Y+(Height-1));
Line(X+(Width-1),Y+1,X+(Width-1),Y+(Height-1)); 

SetColor(0x9BEF,0);
Rectangle(X+1,Y+1,Width-2,Height-2,0);

SetColor(0x001F,0);								
Rectangle(X+2,Y+2,Width-4,14,1);
SetColor(0xFFFF,0x001F);
PutStr(X+4,Y+2,header);

SetColor(0x9BEF,0);
Line(X+1,Y+16,X+(Width-2),Y+16);
//--------client region----------------------------------------
SetColor(0,0);	
Rectangle(X+2,Y+17,(Width-4),Height-19,0);
SetColor(0xFFFF,0);
Rectangle(X+3,Y+18,(Width-6),(Height-21),1);
}


 